//  VideoCameraViewController.m

#import "VideoCameraViewController.h"


@implementation VideoCameraViewController

- (void)viewDidLoad
{
    // ビデオキャプチャデバイスの取得
    AVCaptureDevice*    device;
    device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    
    // デバイス入力の取得
    AVCaptureDeviceInput*   deviceInput;
    deviceInput = [AVCaptureDeviceInput deviceInputWithDevice:device error:NULL];
    
    // ビデオデータ出力の作成
    NSMutableDictionary*        settings;
    AVCaptureVideoDataOutput*   dataOutput;
    settings = [NSMutableDictionary dictionary];
    [settings setObject:[NSNumber numberWithInt:kCVPixelFormatType_32BGRA] 
            forKey:(id)kCVPixelBufferPixelFormatTypeKey];
    dataOutput = [[AVCaptureVideoDataOutput alloc] init];
    [dataOutput autorelease];
    dataOutput.videoSettings = settings;
    [dataOutput setSampleBufferDelegate:self queue:dispatch_get_main_queue()];
    
    // セッションの作成
    _session = [[AVCaptureSession alloc] init];
    [_session addInput:deviceInput];
    [_session addOutput:dataOutput];
  //  _session.sessionPreset = AVCaptureSessionPreset352x288;
    _session.sessionPreset = AVCaptureSessionPreset640x480;
    
    // セッションの開始
    [_session startRunning];
}


//-------------------------------------------------------------
// ucharデータをOpenCV形式の画像(グレースケール)に変換
//-------------------------------------------------------------
cv::Mat * cvMatFrombufGray(unsigned char*in, int w, int h)
{
    cv::Mat *cvMat = new cv::Mat(h, w, CV_8UC1);

    int sptr = 0, ptr = 0;
    for(int x = 0;x < w;x++){
        for(int y = 0;y < h;y++, ptr++, sptr+=4){
            cvMat->data[ptr] = fmin(fmax((0.11*in[sptr] + 0.59*in[sptr+1] + 0.30*in[sptr+2]), 0), 255);
        }
    }
    return cvMat;
}


//--------------------------------------------------------------//
#pragma mark -- AVCaptureVideoDataOutputSampleBufferDelegate --
//--------------------------------------------------------------//
- (void)captureOutput:(AVCaptureOutput*)captureOutput 
        didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer 
        fromConnection:(AVCaptureConnection*)connection
{
    
    // イメージバッファの取得
    CVImageBufferRef    buffer;
    buffer = CMSampleBufferGetImageBuffer(sampleBuffer);
    
    // イメージバッファのロック
    CVPixelBufferLockBaseAddress(buffer, 0);
    
    // イメージバッファ情報の取得
    uint8_t*    base;
    size_t      width, height, bytesPerRow;
    base = (uint8_t*)CVPixelBufferGetBaseAddress(buffer);
    width   = CVPixelBufferGetWidth(buffer);
    height  = CVPixelBufferGetHeight(buffer);
    bytesPerRow = CVPixelBufferGetBytesPerRow(buffer);
    
    //*******************************************************************
    //  OpenCV画像処理ルーチン
    
    // キャプチャ画像から画像バッファを取得
    unsigned char* img = (unsigned char*)CVPixelBufferGetBaseAddress(buffer);
    // OpenCV形式の画像に変換
    cv::Mat *cimg      = cvMatFrombufGray(img,width,height);
    //--------------------------
    // ここの画像処理部分を書く
    cv::Canny(*cimg, *cimg, 20, 20);
    //--------------------------
    //  結果はimgを書き換える
    for(int ptr = 0;ptr < width * height;ptr++){
        img[ptr*4]=img[ptr*4+1]=img[ptr*4+2] = cimg->data[ptr];
    }
    //*******************************************************************
    
    
    // ビットマップコンテキストの作成
    CGColorSpaceRef colorSpace;
    CGContextRef    cgContext;
    colorSpace = CGColorSpaceCreateDeviceRGB();
    cgContext = CGBitmapContextCreate(
            base, width, height, 8, bytesPerRow, colorSpace, 
            kCGBitmapByteOrder32Little | kCGImageAlphaPremultipliedFirst);
    CGColorSpaceRelease(colorSpace);
 
    
    // 画像の作成
    CGImageRef  cgImage;
    UIImage*    image;
    cgImage = CGBitmapContextCreateImage(cgContext);
    image = [UIImage imageWithCGImage:cgImage scale:1.0f 
            orientation:UIImageOrientationRight];
    
    
    CGContextRelease(cgContext);
    CGImageRelease(cgImage);


 
    // イメージバッファのアンロック
    CVPixelBufferUnlockBaseAddress(buffer, 0);
        
    // 画像の表示
    _imageView.image = image;
    
}

@end
